import apf
from apf import utils
import sys
import pprint
import json

USAGE = \
"""%s set <section> <key> <value> [destination] | get <section> [key] | show | dump

In first invocation, set config key in section to value and store configuration
in destination, if provided, else in ~/.apfrc.

In the second invocation, print key from config section if given, otherwise
the entire section.

In the third invocation, show all apf settings.

In the fourth invocation, dump apf settings to apfrc in current working directory.
""" % (sys.argv[0],)

def usage():
    sys.stderr.write(USAGE)
    sys.exit()

def set_config(section, key, value, destination=None):
    apf._set_option(section, key, json.loads(value))
    utils.save_config(destination)

def show_config():
    pprint.pprint(apf._SETTINGS)

def get_setting(section, key=None):
    if key is None:
        pprint.pprint(apf._SETTINGS[section])
    else:
        pprint.pprint(apf._get_option(section, key))

def dump():
    utils.save_config("apfrc")

_COMMANDS = {
    "help": usage,
    "usage": usage,
    "set": set_config,
    "show": show_config,
    "get": get_setting,
    "dump": dump
}

def main():
    if len(sys.argv) < 2:
        usage()
    else:
        command = sys.argv[1]
        if command not in _COMMANDS:
            usage()
        else:
            _COMMANDS[command](*sys.argv[2:])
