import apf
from apf.sims import simulation, plot, generate
from apf.lib.ccalcs import ForceLaw, PotentialTerm
from apf import movie, utils
import os
import sys

def _make_prefix():
    path = apf._get_option("fullsim", "pngs")
    prefix = apf._get_option("fullsim", "prefix")
    if not os.path.isdir(path):
        os.mkdir(path)
    prefix = os.path.join(path, prefix)
    return prefix

def full_sim_simulate():
    params = simulation.SimulationParameters.from_config()
    sys.stderr.write("Taking input from %s and dumping to %s.\n" % (params.infile, params.outfile))
    sys.stderr.write("Using timestep %.3f over %s steps.\n" % (params.timestep, params.steps))
    sim = simulation.Simulation(params)
    sim.run()
    sys.stderr.write("Simulation done...\n")

def full_sim_plot():
    infile = apf._get_option("fullsim", "output") # Output from simulation
    processes = apf._get_option("plot", "processes")
    sys.stderr.write("Plotting from infile %s, using %s processes.\n" % (infile, processes))
    try:
        plot.run(_make_prefix(), plot.get_framegroups(infile, processes), processes)
    except IOError: # This happens on Windows for whatever reason
        sys.stderr.write("Failed to plot with multiprocessing, falling back to single process...\n")
        plot.plot_file(_make_prefix(), infile)
    
def full_sim_movie():
    output = apf._get_option("fullsim", "movie")
    movie._main(_make_prefix(), output)

def _create(name):
    if not os.path.isdir(name):
        os.mkdir(name)
    _old = os.getcwd()
    os.chdir(name)
    _prepare_project()
    os.chdir(_old)

def create():
    if len(sys.argv) < 2:
        sys.stderr.write("Usage: %s <directory> - start apf project in directory.\n" % sys.argv[0])
    else:
        _create(sys.argv[1])
        sys.stderr.write("Generated new apf project in %s\n" % (sys.argv[1]))

def _dump_visualize():
    with open("script.py", "w") as out:
        with utils.get_apf_data_file("viz") as script:
            for line in script:
                out.write(line)
                
def _prepare_project():
    utils.save_config("apfrc")
    _generate()
    _dump_visualize()
    _make_prefix()

def _generate():
    bg = generate.BoxGenerator(*apf._get_option("generate", "box"))
    bg.min_distance(apf._get_option("generate", "mindistance"))
    bg.internal = ForceLaw([PotentialTerm(20, 2), PotentialTerm(-1, 1)])
    out = bg.group_manager(apf._get_option("generate", "units")).to_json()
    with open(apf._get_option("fullsim", "in"), "wb") as outfile:
        outfile.write(out)
    
def main():
    sys.stderr.write("Running full simulation as specified in apfrc - this might take a while.\n")
    full_sim_simulate()
    full_sim_plot()
    full_sim_movie()
