"""
Some simple geometric calculations.
"""

from apf.lib import ccalcs
import math

from apf.lib.ccalcs import bounding_box, bounding_box_side_lengths, center_of_mass, objects_near

class Triangle(object):

    def __init__(self, a, b, c):

        self.a = a
        self.b = b
        self.c = c
        self._area = None

    def _calc_area(self):

        ab, ac = self.b - self.a, self.c - self.a
        dotted = ab.dot(ac)
        abl, acl = ab.length(), ac.length()
        angle = math.acos(dotted / (abl * acl))
        self._area = 0.5 * (acl * abl * math.sin(angle))
        
    def area(self):

        if self._area is None:
            self._calc_area()
        return self._area

    def contains(self, point):

        if point in (self.a, self.b, self.c):
            return True
        if self._area is None:
            self._area = self.area()
        t1, t2, t3 = (Triangle(self.a, self.b, point),
                      Triangle(self.a, self.c, point),
                      Triangle(self.b, self.c, point))
        try:
            return abs(sum(map(Triangle.area, [t1, t2, t3])) - self._area) < 1e-6
        except ZeroDivisionError:
            # ab or ac vectors had length of 0 - point is clearly in triangle
            return True
        except ValueError:
            # two of the lines in an attempted triangle were parallell
            return True

    def rotate(self, angle):

        center = center_of_mass([self.a, self.b, self.c])
        self.a -= center
        self.b -= center
        self.c -= center
        self.a.irotate(angle)
        self.b.irotate(angle)
        self.c.irotate(angle)
        self.a += center
        self.b += center
        self.c += center
        
    def move(self, direction):

        self.a += direction
        self.b += direction
        self.c += direction

    def center_of_mass(self):
        return center_of_mass([self.a, self.b, self.c])
        