import apf.lib.ccalcs as c
import unittest
import pprint

class TestVector(unittest.TestCase):

    def setUp(self):

        self.b = c.Vector2(1.7, 9.2)
        self.a = c.Vector2(4.3, 7.2)

    def test_add_and_iadd(self):
        r = self.a + self.b
        self.a += self.b
        self.assertEquals(r, self.a)
        with self.assertRaises(TypeError):
            self.a + 4
        with self.assertRaises(TypeError):
            self.a += 4

    def test_sub_and_isub(self):
        r = self.a - self.b
        self.a -= self.b
        self.assertEquals(r, self.a)
        with self.assertRaises(TypeError):
            self.a - 4
        with self.assertRaises(TypeError):
            self.b -= 4.7

    def test_length_and_lengthSquared(self):
        self.assertTrue(self.b.length_squared() > self.a.length_squared())
        self.assertTrue(self.b.length() > self.a.length())

    def test_richcmp(self):
        self.assertTrue(self.a != self.b)
        self.assertTrue(self.a < self.b)

    def test_neg(self):

        self.assertTrue((-self.a).x < 0)
        self.assertTrue((-self.b).y < 0)

    def test_divide_and_normalize(self):

        self.assertTrue(abs(1 - self.a.normalize().length()) < 0.00001)
        self.assertTrue(abs(1 - self.a.inormalize().length()) < 0.00001)

    def test_angle(self):

        self.assertTrue(0 <= self.a.angle() <= 1.57)
        self.assertTrue(0 <= self.a.sin() <= 1.57)
        self.assertTrue(0 <= self.a.rsin() <= 1.57)
        self.assertTrue(0 <= self.b.cos() <= 1.57)
        self.assertTrue(0 <= self.b.rcos() <= 1.57)
        
    def test_rotate(self):

        r = self.a.rotate(1.57)
        self.assertTrue(abs(r.length() - self.a.length()) < 0.00001)
        r = self.b.irotate(1.57)
        self.assertEquals(self.b, r)

    def test_dot_and_perpendicular(self):

        self.assertTrue(self.a.dot(self.a) > 0)
        self.assertTrue(abs(self.a.dot(self.a.perpendicular())) < 0.00001)

    def test_scale(self):
        self.assertTrue(self.a.scale(3).length() / self.a.length() >= 3 - 1e-6)
        length = self.b.length()
        self.assertTrue(self.b.iscale(3).length() / length >= 3 - 1e-6)
        
    def test_dict(self):
        self.assertTrue(isinstance(self.a.dict(), dict))
        self.assertTrue(self.a.dict().has_key('x') and self.a.dict().has_key('y'))

    def test_iscale_down(self):
        r = self.a.scale_down(2)
        self.assertTrue(r.length() <= 2)
        self.assertTrue(self.a.length() > r.length())
        r = self.a.iscale_down(2)
        self.assertTrue(self.a.length() <= 2)
        
class TestSimObject(unittest.TestCase):

    def setUp(self):
        self.point = c.SimObject(c.Vector2(0, 0))
        self.line = c.SimObject(c.Vector2(1, 2), c.Vector2(4, 2))

    def test_isLine(self):
        self.assertTrue(not self.point.isLine())
        self.assertTrue(self.line.isLine())

    def test_centerOfMass(self):
        self.assertEquals(self.point.centerOfMass(c.Vector2(3, 2)).vector, c.Vector2(0, 0))
        self.assertEquals(self.line.centerOfMass(c.Vector2(3, 3)).vector, c.Vector2(3, 2))
        self.assertEquals(self.line.centerOfMass(c.Vector2(1, 2)).isVector, 0)

    def test_distance(self):
        self.assertEquals(self.point.distance(c.ONE).vector, c.ONE)
        self.assertEquals(self.line.distance(c.Y + self.line.start).vector, c.Y)
        self.assertEquals(self.line.distanceRoundEdges(c.Vector2(-1, 2)).vector.length(), 2.0)
        
    def test_dict(self):
        self.assertTrue(isinstance(self.line.dict(), dict))
        self.assertEquals(self.line.dict(), c.dict_to_sim_object(self.line.dict()).dict())

if __name__ == "__main__":

    unittest.main()
    