import apf.sims.groups as G
import apf.lib.ccalcs as C
from apf import utils
import unittest
import json
import sys
import pprint

class TestGroup(unittest.TestCase):

    def setUp(self):

        self.objects = [C.SimObject(C.Vector2(1, 2), C.Vector2(0, 1)),
                        C.SimObject(C.Vector2(3, 3)),
                        C.SimObject(C.Vector2(-1, -2))]
        self.internal = C.ForceLaw([C.PotentialTerm(1, 2),
                                    C.PotentialTerm(-.5, 1)])
        self.external = C.ForceLaw([C.PotentialTerm(30, 1),
                                    C.PotentialTerm(-5, 3)])
        self.group = G.SimObjectGroup(self.objects, self.internal, self.external)
        self.other = G.SimObjectGroup([C.SimObject(C.Vector2(4, 3))], C.ForceLaw([]), C.ForceLaw([]))
        
    def testInternal(self):

        self.group.calcInternalForces()
        for obj in self.objects:
            self.assertTrue(obj.data.now != C.ZERO)

    def testExternal(self):

        self.group.affectOther(self.other)
        self.assertTrue(self.other.objects[0].data.now != C.ZERO)

    def testDict(self):
        self.assertTrue(isinstance(G.SimObjectGroup.from_dict(self.group.dict()), G.SimObjectGroup))

    def testJson(self):
        self.assertTrue(isinstance(G.SimObjectGroup.from_json(self.group.to_json()), G.SimObjectGroup))

    def testMove(self):

        self.group.affectOther(self.other)
        self.assertTrue(self.other.objects[0].data.now != C.ZERO)
        now = self.other.objects[0].refpoint().copy()
        self.other.moveAllObjects(0.2)
        self.assertTrue(now != self.other.objects[0].refpoint())

class TestGroupManager(unittest.TestCase):

    def setUp(self):
        self.content = utils.get_apf_data_file("testrun.json").read()
        self.groups = G.GroupManager.from_json(self.content)

    def testLoad(self):
        self.assertTrue(isinstance(self.groups, G.GroupManager))

    def testDump(self):
        self.groups.dump(sys.stdout)

    def loadAgain(self):
        return G.GroupManager.from_json(self.content)

    def testCalcForces(self):
        old = self.loadAgain()
        self.groups.calcForces()
        for group in self.groups.groups.values():
            for index, obj in enumerate(group.objects):
                oldobj = old.groups[group.tag].objects[index]
                if group.tag in [target for (source, target) in
                                 self.groups.relations]:
                    self.assertTrue(obj.data.now != oldobj.data.now)
                
if __name__ == "__main__":

    unittest.main()
