"""
Utility module for finding data files and easily running apf commands from python.
"""
import sys
import os
import apf
import json

# This module is in the root of the apf package. There should be a data/
# here, for storing input to and output from simulation.py.

def locate_package_dir(pkg):
    path, init = os.path.split(os.path.abspath(pkg.__file__))
    return path

def get_pkg_dir(pkg, dirname):
    return os.path.join(locate_package_dir(pkg), dirname)

DATA_DIR = get_pkg_dir(apf, "data")

def get_apf_data(datafile, path=DATA_DIR):
    return os.path.join(DATA_DIR, datafile)

def get_apf_data_list(suffix, path=DATA_DIR):
    fnames = [os.path.split(fname)[1] for fname in os.listdir(path)]
    return [fname for fname in fnames if fname.endswith(suffix)]

def get_apf_data_file(datafile):
    return open(get_apf_data(datafile), "rb")

def _load_config(fname):
    with open(fname, "rb") as config_file:
        content = config_file.read()
        return json.loads(content)

def _home_dir():
    return os.path.expanduser("~")

def _update_settings(settings, loaded):
    for key in loaded:
        settings[key].update(loaded[key])
        
def _read_config():
    pkg_dir = locate_package_dir(apf)
    global_file = os.path.join(pkg_dir, "settings.json")
    global_settings = _load_config(global_file)
    user_file = os.path.join(_home_dir(), ".apfrc")
    try:
        _update_settings(global_settings,
                         (_load_config(user_file)))
    except IOError:
        pass
    try: # Settings for current folder
        _update_settings(global_settings,
                         _load_config(os.path.join(os.getcwd(), "apfrc")))
    except IOError:
        pass
    return global_settings

def save_config(destination=None):
    """Store configuration data (in full) to destination. If None, use ~/.apfrc."""
    if destination is None:
        destination = os.path.join(_home_dir(), ".apfrc")
    with open(destination, "wb") as config_file:
        config_file.write(
            json.dumps(apf._get_settings(), indent=apf._get_option("all", "indent")))

